#ifndef STARS_IRDB_Op_h
#define STARS_IRDB_Op_h

#include <stdint.h>
#include <assert.h>
#include "interfaces/STARSTypes.h"
#include <libIRDB-core.hpp>
#include "interfaces/abstract/STARSOp.h"
#include <string.h>

// forward decls
class STARS_IRDB_op_t;

// to help easy porting if this ever changes.
typedef std::shared_ptr<STARS_IRDB_op_t> STARSIRDBOpndTypePtr;

class STARS_IRDB_op_t : public STARS_op_t
{
	public:
		
		// Constructors
		STARS_IRDB_op_t() { assert(0); }
		STARS_IRDB_op_t(const DISASM &d, int indx, const ARGTYPE &the_arg) 
			: disasm(d), index(indx), arg(the_arg) {  }

		// Clone method (deep copy).
		virtual STARSOpndTypePtr clone(void) const 
		{ 
			STARSOpndTypePtr p=std::make_shared<STARS_IRDB_op_t>(disasm,0,arg);
			return p;
		}

		// Operators
		virtual bool operator<(const STARS_op_t &rOp) const { assert(0); } // Less-than operator for use in STL ordered containers, e.g. sets.

		// Get methods
		virtual unsigned int GetOpGlobalIndex(void) const { assert(0); } // Get STARS SSA name index.
		virtual STARS_ea_t GetAddr(void) const { assert(0); }
		virtual unsigned char GetOpType(void) const { assert(0); } // Get type o_reg, o_displ, etc.
		virtual uint16_t GetReg(void) const { assert(0); }  // Get reg field of operand, whether it is an addressing reg or directly used reg in register operand
		virtual uint16_t GetSegReg(void) const { assert(0); } // Get segment register of operand; R_none if no segment register
		virtual char GetSIB(void) const { assert(0); } // Get x86 SIB byte (dense encoding of base reg, index reg, and scale factor)
		virtual int MDGetSIBBaseReg(void) const { assert(0); } // Get x86 SIB byte base register
		virtual short MDGetSIBIndexReg(void) const { assert(0); } // Get x86 SIB byte index register
		virtual int GetSIBScaleFactor(void) const { assert(0); } // Extract scale factor from SIB byte
		virtual char GetSpecFlag4(void) const { assert(0); } // Get specflag4 byte, used to hold a copy of the auxpref byte in x86-64 programs.
		virtual STARS_uval_t GetImmedValue(void) const { assert(0); } // Get value field for immediate operands; uint32 for x86-32, uint64 for x86-64
		virtual char GetOpDtyp(void) const { assert(0); }  // Get field that determines byte width
		virtual uint16_t GetByteWidth(void) const { assert(0); }

		// Set methods
		virtual void SetByteWidth(uint16_t ByteWidth) { assert(0); } // Change default byte width of opnd
		virtual void DoubleRegWidth(void) { assert(0); } // Double the width of opnd if it is a subword register
		virtual void SetOpGlobalIndex(std::size_t index) { assert(0); } // Set STARS SSA name index.
		virtual void SetReg(uint16_t NewReg) { assert(0); }
		virtual void SetAddr(STARS_ea_t NewAddr) { assert(0); }
		virtual void SetSIB(char value) { assert(0); }
		virtual void SetSpecFlag4(char value) { assert(0); }
		virtual void SetBitInSpecFlag4(char value) { assert(0); } // OR in the value to set a bit
		virtual void SetNotVisible(void) { assert(0); } // Set operand to type that is implicit, not explicit in disasm
		virtual void SetTypeToMemDisplacement(void) { assert(0); } // change type during stack normalization

		// Query methods
		virtual bool IsVisible(void) const { assert(0); }  // Operand appears in disasm, vs. being implicit, e.g. "imul ecx" also has eax opnd
		virtual bool IsRegOp(void) const { assert(0); }
		virtual bool IsImmedOp(void) const { assert(0); }
		virtual bool IsVoidOp(void) const { assert(0); }
		virtual bool IsMemDisplacementOp(void) const { assert(0); }
		virtual bool IsStaticMemOp(void) const { assert(0); }
		virtual bool IsMemNoDisplacementOp(void) const { assert(0); }
		virtual bool IsMemOp(void) const { assert(0); }  // any of the previous three types
		virtual bool HasSIBByte(void) const { assert(0); }
		virtual bool IsFloatingPointRegOp(void) const { assert(0); }
		virtual bool IsMMXRegOp(void) const { assert(0); }
		virtual bool IsXMMRegOp(void) const { assert(0); }
		virtual bool IsYMMRegOp(void) const { assert(0); }
		virtual bool IsTestRegOp(void) const { assert(0); }
		virtual bool IsDebugRegOp(void) const { assert(0); }
		virtual bool IsControlRegOp(void) const { assert(0); }
		virtual bool MDIsKnownOpType(void) const { assert(0); }
		virtual bool MDIsSpecialRegOpType(void) const { assert(0); }  // special debug reg, control or task reg, MMX/XMM/YMM registers
		virtual bool MatchesReg(uint16_t RegNum) const { assert(0); }
		virtual bool IsNearPointer(void) const { assert(0); }
		virtual bool IsFarPointer(void) const { assert(0); }
		virtual bool HasSegReg(void) const { assert(0); } // Has a segment register


// conversion of op to new type
                virtual void MakeVoidOpnd(void)  { memset(&arg,0,sizeof(arg)); }
                virtual void MakeImmediateOpnd(STARS_uval_t value)  { assert(0); }
                virtual void MakeRegOpnd(uint16_t RegNum)  { assert(0); }
                virtual void MakeFloatingPointRegOpnd(uint16_t RegNum)  { assert(0); }
                virtual void MakeMMXRegOpnd(uint16_t RegNum)  { assert(0); }
                virtual void MakeXMMRegOpnd(uint16_t RegNum)  { assert(0); }
                virtual void MakeYMMRegOpnd(uint16_t RegNum)  { assert(0); }
                virtual void MakeMemDisplacementOpnd(uint16_t BaseRegNum, 
                        uint16_t IndexRegNum, uint16_t ScaleFactor, STARS_ea_t offset)  { assert(0); }
                virtual void MakeMemPhraseOpnd(uint16_t BaseRegNum, 
                        uint16_t IndexRegNum, uint16_t ScaleFactor)  { assert(0); }


	private:
		DISASM disasm;
		ARGTYPE arg;
		int index;

};

#endif



